#-------------------------------------------------------------------------------------------------------
# 配付ファイル
## 欠測値のあるデータ：example.csv
## 完全データ：complete.csv
#-------------------------------------------------------------------------------------------------------
# パッケージのインストール（必要に応じてパッケージをインストールしてください）														
## install.packages("lavaan")
## install.packages("mice")	
## install.packages("mitml")
## install.packages("multcomp")
## install.packages("semTools")
#-------------------------------------------------------------------------------------------------------

# データの読み込み
dat_mis <- read.csv(file.choose(), na.strings = ".") # フォルダからexample.csv を選択

# 最初の8行を表示
head(dat_mis, 8)


#-------------------------------------------------------------------------------------------------------
#     多重代入法による欠測の補完
#-------------------------------------------------------------------------------------------------------

# パッケージの読み込み
library(mice)

# 欠測状況の確認
md.pattern(dat_mis)

# 欠測の補完
dat_MI <- mice(data = dat_mis, method = "pmm", m = 100, maxit = 30, seed = 23109, printFlag = FALSE)

# 多重代入の方法の確認
dat_MI$method

# 代入値の確認
## 変数y1に代入された値の確認
dat_MI$imp$y1
## 3～11列目の変数に代入された値の確認
dat_MI$imp[c(3:11)]

# 疑似完全データセットの確認
## 1つ目の疑似完全データの確認
complete(dat_MI, 1)
## 3つ目の疑似完全データの最初の8行の確認
head(complete(dat_MI, 3), 8) 


#-------------------------------------------------------------------------------------------------------
#     確認的因子分析
#-------------------------------------------------------------------------------------------------------

# モデルの記述
CFA_model <-　'
F1 =~ y1 + y2 + y3 + y4
F2 =~ y5 + y6 + y7 + y8
'

# パッケージの読み込み
library(lavaan)

# FIML法による確認的因子分析
## 母数推定
CFA_FIML <- cfa(CFA_model, data = dat_mis, missing = "fiml", std.lv = TRUE) 
## 結果の出力
summary(CFA_FIML, fit.measures = TRUE, standardized = TRUE, ci = TRUE)


# 性別とテスト得点を補助変数とするFIML法による確認的因子分析
## パッケージの読み込み
library(semTools)
## 母数推定
CFA_FIML_aux <- cfa.auxiliary(CFA_model, data = dat_mis, missing = "fiml", std.lv = TRUE, aux = c("gender", "score")) 
## 結果の出力
summary(CFA_FIML_aux, fit.measures = TRUE, standardized = TRUE, ci = TRUE)


# 多重代入法による確認的因子分析
## 母数推定
CFA_MI <- cfa.mi(CFA_model, data = dat_MI, std.lv = TRUE)
## 結果の出力
summary(CFA_MI, fit.measures = TRUE, standardized = TRUE, ci = TRUE, output = "data.frame")


#-------------------------------------------------------------------------------------------------------
#     探索的因子分析
#-------------------------------------------------------------------------------------------------------

# FIML法による探索的因子分析
## 探索的因子分析（2因子解・オブリミン回転）
EFA_FIML <- efa(dat_mis[c(3:10)], nfactors = 2, rotation = "oblimin", missing = "fiml")
## 結果の出力
summary(EFA_FIML, cutoff = 0)
## 適合度指標の出力
fitMeasures(EFA_FIML)


#-------------------------------------------------------------------------------------------------------
#     回帰分析
#-------------------------------------------------------------------------------------------------------

# 合成変数の作成
dat_mis$Int <- rowMeans(dat_mis[c(3:6)])
dat_mis$Ext <- rowMeans(dat_mis[c(7:10)])


# 合成変数の作成（疑似完全データ）
## パッケージの読み込み
library(mitml)
## オブジェクトのコンバート
dat_MI_list <- mids2mitml.list(dat_MI)
## 合成変数の作成
for(i in 1:100){
 dat_MI_list[[i]]$Int <- rowMeans(dat_MI_list[[i]][c(3:6)])
 dat_MI_list[[i]]$Ext <- rowMeans(dat_MI_list[[i]][c(7:10)])
}


# モデルの記述
reg_model <-　'
score ~ Int + Ext
Int ~~ Int
Ext ~~ Ext
Int ~~ Ext
'


# FIML法による回帰分析
## 母数推定
reg_FIML <- sem(reg_model, data = dat_mis, missing = "fiml") 
## 結果の出力
summary(reg_FIML, standardized = TRUE, rsquare = TRUE, ci = TRUE)


# MI法による回帰分析
## 母数推定
reg_MI <- sem.mi(reg_model, data = dat_MI_list)
## 結果の出力
summary(reg_MI, standardized = TRUE, rsquare = TRUE, ci = TRUE, output = "data.frame")


# 関数with()を用いた回帰分析（MI法）
## 分析
reg_MI_lm <- with(dat_MI_list, lm(score ~ Int + Ext))
## 結果の統合と出力
summary(pool(reg_MI_lm))
## 決定係数の出力
est <- pool(reg_MI_lm)
### 決定係数
pool.r.squared(est)
### 自由度調整済み決定係数
pool.r.squared(est, adjusted = TRUE)


#-------------------------------------------------------------------------------------------------------
#     t検定
#-------------------------------------------------------------------------------------------------------

# モデルの記述
t_model <-　'
score ~ gender
gender ~~ gender
'


# FIML法によるt検定
## 母数推定
t_FIML <- sem(t_model, data = dat_mis, missing = "fiml")
## 結果の出力
summary(t_FIML, ci = TRUE)


# MI法によるt検定
## 母数推定
t_MI <- sem.mi(t_model, data = dat_MI_list)
## 結果の出力
summary(t_MI, ci = TRUE, output = "data.frame")


# 関数with()を用いたt検定（MI法）
## 分析
t_MI_lm <- with(dat_MI_list, lm(score ~ gender))
## 結果の統合と出力
summary(pool(t_MI_lm))


#-------------------------------------------------------------------------------------------------------
#     分散分析
#-------------------------------------------------------------------------------------------------------

# 学校種を要因型に変換
for(i in 1:100){
 dat_MI_list[[i]]$school <- as.factor(dat_MI_list[[i]]$school)
}


# 分散分析
anova_MI <- with(dat_MI_list, lm(score ~ school))
## 結果の統合
testEstimates(anova_MI)


# 多重比較
## パッケージの読み込み
library(multcomp)
## 分析
anova_MI_pairwise <- lapply(anova_MI, glht, linfct = mcp(school = "Tukey"))
##　結果の統合と出力
testEstimates(anova_MI_pairwise)
